#!/usr/bin/env python
# amaroK dcop interface

import os,pprint,re,md5

class amaroKdcop:
   debug=0
   pp = pprint.PrettyPrinter()
   
   def __init__(self,database=0):
      """amazonCovers: path to amazon covers dir
                      eg: /home/jjl/.kde/share/apps/amarok/albumcovers/
         database: 0:SQLite
                   1:MySQL"""
      self.db=database
      if self.debug: print "database:"+self.db
      #self.db=1

   def getArtists (self):
      """return hash of all artists"""
      return dict(self.execSql("select id||'##'||name from artist"))

   def getArtistAlbums(self,artist):
      """Get all albums of an artist
      in: artist id
      return dict id:name"""
      return dict(self.execSql("select t.album||'##'||a.name from album a,tags t where t.artist=\"%s\" and t.album=a.id"%(artist)))

   def getAlbums(self):
      """return hash of all albums"""
      return dict(self.execSql("select id||'##'||name from album"))

   def getAlbumContent(self,id,amazonCoverFolder):
      """return album structure"""
      struct={}
      list = self.execSql("select track||'##'||title||'##'||length||'##'||url from tags where album=\"%s\" order by track"%(id,))
      if len(list[0][0]) == 0:
         # list is empty, maybe tags are missing some information
         # try a smalest query
         list = self.execSql("select '0##'||title||'##'||length||'##'||url from tags where album=%s "%(id,))
      struct['tracks']=list
      struct['artist']=self.execSql("select artist from tags t where t.album=%s LIMIT 1"%(id))[0][0]
      struct['album_name']=self.execSql("select name from album a where a.id=%s "%(id))[0][0]
      struct['artist_name']=self.execSql("select name from artist a where a.id=%s "%(struct['artist']))[0][0]
      struct['cover']=self.getImage(struct['artist_name'],struct['album_name'],amazonCoverFolder)
      struct['id']=id
      return struct

   def getImage(self,artist,album,acf):
      """return path to image of album
      search into amazon folder (acf) then image table
      return "" or path containing front or first found path"""
      
      def key (artist,album):
         """return key for artist,album
         used to search into amazon table for downloaded image"""
         return md5.new(artist.lower() + album.lower()).hexdigest()
      
      # search for amazon cover
      amazoncover=os.path.join(acf,"large",key(artist,album))
      if (os.path.isfile(amazoncover)):
         if self.debug: print "Cover from amazon: "+amazoncover
         return amazoncover
      # search registered image
      list = self.execSql("""select path from images where images.artist="%s" and images.album="%s" """ % (artist,album) )
      # if no cover
      if list[0][0] == "":return ""
      # search for a name containing front
      p = re.compile(".*front.*",re.IGNORECASE)
      front = [l for l in list if p.match(l[0])]
      if self.debug==2: print "Cover from dir "+str(list)
      return len(front)>=1 and front[0][0] or list[0][0]

   #def getImage(self,id):
      #"""return path to image of album
      #search into image table
      #return "" or path containing front or first found path"""
      #list = self.execSql("select path from images, album a where images.album = a.name and a.id=%s"%(id,))
      ## if no cover
      #if list[0][0] == "":return ""
      ## search for a name containing front
      #p = re.compile(".*front.*",re.IGNORECASE)
      #front = [l for l in list if p.match(l[0])]
      #return len(front)>=1 and front[0][0] or list[0][0]

   def getSongs (self):
      """return list of all songs
      (title,length,url,album,artist)"""
      #print dict(self.execSql("select id||'##'||name from artist"))
      return self.execSql("select title||'##'||length||'##'||url||'##'||album||'##'||artist from tags")
      
   def getStats(self):
      """gather db stats in a dict"""
      stats={}
      stats['artists'] = self.execSql("select count(1) from artist")[0][0]
      stats['albums'] = self.execSql("select count(1) from album")[0][0]
      stats['songs'] = self.execSql("select count(1) from tags")[0][0]
      return stats
   
   def execSql (self,requete):
      """execute SQL request to amarok
         requete : string : the SQL request
                  values must be between ", not '
                  select * from toto where titi="string'toto"
         return lines in a list"""
      #escape " character
      requete = requete.replace("\"","\\\"")
      if self.db=="1" and requete.find("||")!=-1:
         # MySQL : we need to replace standard SQL operator ||
         #   by function concat !
         # Why can't MySQL respect SQL standard ?
         requete = requete.replace("select ","select concat(").replace("||",",").replace(" from",") from")
      if self.debug: print "execSql(db=%s): %s"%(self.db,requete)
      ret=[]
      # exec the SQL request
      stdout = os.popen("dcop amarok collection query \"%s\""%requete)
      line = stdout.readline()
      while line:
         if self.debug==2: print '<'+line.rstrip("\n")+'>'
         ret.append(line.rstrip('\n').split('##'))
         line = stdout.readline()
      if self.debug: self.pp.pprint(ret)
      return ret

if __name__=="__main__":
   a = amaroKdcop()
   #print a.getArtists()
   #a.pp.pprint(a.getAlbums())
   #a.getAlbumContent(5)
   #print a.getImage(2)
   print a.getImage("Ludwig Von 88","45 t les trois p'tits keupons")
   
